@@ -24,8 +24,8 @@ struct CliqueTree{V, E} <: AbstractVector{Clique{V, E}}
     graph::BipartiteGraph{V, E, FVector{E}, FVector{V}}
 
     function CliqueTree{V, E}(tree::SupernodeTree{V}, graph::BipartiteGraph{V, E}) where {V, E}
-        @argcheck nv(residuals(tree)) == nv(graph)
-        @argcheck nov(residuals(tree)) == nov(graph)
+        @assert nv(residuals(tree)) == nv(graph)
+        @assert nov(residuals(tree)) == nov(graph)
         return new{V, E}(tree, graph)
     end
 end
@@ -94,16 +94,16 @@ function cliquetree(graph, alg::PermutationOrAlgorithm, snd::SupernodeType)
     return cliquetree(BipartiteGraph(graph), alg, snd)
 end
 
-function cliquetree(graph::AbstractGraph{V}, alg::PermutationOrAlgorithm, snd::SupernodeType) where {V}
+function cliquetree(graph::AbstractGraph{V}, alg::PermutationOrAlgorithm, snd::SupernodeType = Maximal()) where {V}
     n = nv(graph); weights = Ones{V}(n)
     return cliquetree(weights, graph, alg, snd)
 end
 
-function cliquetree(weights::AbstractVector, graph, alg::PermutationOrAlgorithm, snd::SupernodeType)
+function cliquetree(weights::AbstractVector, graph, alg::PermutationOrAlgorithm, snd::SupernodeType = Maximal())
     return cliquetree(weights, BipartiteGraph(graph), alg, snd)
 end
 
-function cliquetree(weights::AbstractVector, graph::AbstractGraph{V}, alg::PermutationOrAlgorithm, snd::SupernodeType) where {V}
+function cliquetree(weights::AbstractVector, graph::AbstractGraph{V}, alg::PermutationOrAlgorithm, snd::SupernodeType = Maximal()) where {V}
     E = etype(graph); n = nv(graph); m = half(de(graph))
     nn = n + one(V); nnn = nn + one(V)
     
@@ -131,7 +131,9 @@ function cliquetree(weights::AbstractVector, graph::AbstractGraph{V}, alg::Permu
         elmorder, elmindex, sndptr, sepptr, new, parent, elmtree, graph,
         order, index, snd)
 
-    h = last(sndtree.tree); k = sepptr[h + one(V)] - one(E)
+    h = last(sndtree.tree)
+    k = sepptr[h + one(V)] - one(E)
+
     septgt = FVector{V}(undef, k)
     separator = BipartiteGraph(n, h, k, sepptr, septgt)
 
@@ -243,7 +245,7 @@ function cliquetree(tree::CliqueTree{V, E}, root::Integer) where {V, E}
     q = h + one(V); pointers(residual)[n + one(V)] = q
 
     sndtree = copy(tree.tree.tree.tree)
-    sndinvp = postorder!(setrootindex!(sndtree, root))
+    sndinvp = postorder!(setrootindex!(sndtree, root), root)
     sndperm = invperm(sndinvp)
 
     for v in oneto(h)
@@ -275,12 +277,238 @@ function cliquetree(tree::CliqueTree{V, E}, root::Integer) where {V, E}
     return collect(perm), tree
 end
 
+function cliquetree(tree::CliqueTree{V, E}, clique::AbstractVector{V}) where {V, E}
+    sep = separators(tree)
+    res = residuals(tree)
+
+    h = nov(sep)
+    n = nv(sep)
+    m = ne(sep)
+    k = convert(V, length(clique))
+
+    mask = FVector{V}(undef, h)
+    invp = FVector{V}(undef, h)
+
+    for v in outvertices(sep)
+        mask[v] = zero(V)
+    end
+
+    for v in clique
+        mask[v] = one(V)
+    end
+
+    root = n
+
+    for i in vertices(res)
+        root < n && break
+
+        for v in neighbors(res, i)
+            root < n && break
+
+            if ispositive(mask[v])
+                root = i
+            end
+        end
+    end
+
+    perm, newtree = cliquetree(tree, root)
+    newsep = separators(newtree)
+    newres = residuals(newtree)
+    newbag = neighbors(newres, n)
+
+    i = h - k
+    j = h
+
+    for v in reverse(newbag)
+        if !ispositive(mask[perm[v]])
+            invp[v] = i; i -= one(V)
+        else
+            invp[v] = j; j -= one(V)
+        end
+    end
+
+    for v in newbag
+        mask[invp[v]] = perm[v]
+    end
+
+    for v in newbag
+        perm[v] = mask[v]
+    end
+    
+    for p in oneto(m)
+        v = targets(newsep)[p]
+
+        if v in newbag
+            targets(newsep)[p] = invp[v]
+        end
+    end
+
+    return perm, newtree
+end
+
 function cliquetree!(tree::CliqueTree, root::Integer)
-    perm, source = cliquetree(tree, root)
-    copy!(tree, source)
+    perm, newtree = cliquetree(tree, root)
+    copy!(tree, newtree)
     return perm
 end
 
+function cliquetree!(tree::CliqueTree, clique::AbstractVector)
+    perm, newtree = cliquetree(tree, clique)
+    copy!(tree, newtree)
+    return perm
+end
+
+function atomtree(graph::AbstractGraph; alg::MinimalAlgorithm=DEFAULT_MINIMAL_ALGORITHM)
+    return atomtree(graph, alg)
+end
+
+function atomtree(graph::AbstractGraph, alg::MinimalAlgorithm)
+    return atomtree!(cliquetree(graph, alg)..., graph)
+end
+
+function atomtree!(order::AbstractVector{V}, tree::CliqueTree{V, E}, graph::AbstractGraph{V}) where {V, E}
+    res = residuals(tree)
+    sep = separators(tree)
+    
+    h = nov(sep); m = ne(sep); n = nv(sep)
+
+    work1 = FVector{V}(undef, h)
+    work2 = FVector{V}(undef, n)
+    work3 = FVector{V}(undef, n)
+
+    mark = FVector{V}(undef, h)
+    prnt = FVector{V}(undef, n)
+    proj = FVector{V}(undef, n)
+
+    resptr = FVector{V}(undef, n + one(V))
+    sepptr = FVector{E}(undef, n + one(V))
+    septgt = FVector{V}(undef, m)
+
+    for v in vertices(graph)
+        mark[v] = n + one(V)
+    end
+
+    qode = zero(V)
+    
+    for node in reverse(oneto(n))
+        flag = true; maxcnt = zero(V)
+        
+        for i in neighbors(sep, node)
+            v = order[i]
+            mark[v] = node; maxcnt += one(V)
+        end
+
+        for i in neighbors(sep, node)
+            v = order[i]; cnt = maxcnt - one(V)
+
+            for w in neighbors(graph, v)
+                if w != v && mark[w] == node
+                    cnt -= one(V)
+                end
+            end
+
+            if ispositive(cnt)
+                flag = false
+                break
+            end
+        end
+
+        npnt = parentindex(tree, node)
+
+        if isnothing(npnt)
+            spnt = zero(V)
+        else
+            spnt = abs(proj[npnt])
+        end
+
+        if flag
+            proj[node] = qode += one(V)
+            prnt[qode] = spnt
+        else
+            proj[node] = -spnt
+        end
+    end
+
+    q = qode; qree = Parent(q, prnt)
+    postorder!_impl!(work1, work2, mark, work3, qree)
+
+    for node in oneto(n)
+        qode = proj[node]
+
+        if ispositive(qode)
+            qode = mark[qode]
+        else
+            qode = -mark[-qode]
+        end
+            
+        proj[node] = qode
+    end
+
+    for qode in oneto(q + one(V))
+        resptr[qode] = zero(V)
+    end
+
+    sepptr[q + one(V)] = zero(E)
+
+    for node in oneto(n)
+        qode = proj[node]
+        
+        if ispositive(qode)
+            sepptr[qode] = eltypedegree(sep, node)
+        else
+            qode = -qode
+        end
+        
+        resptr[qode] += eltypedegree(res, node)
+    end
+    
+    r = one(V)
+    s = one(E)
+
+    for qode in oneto(q + one(V))
+        resptr[qode] = r += resptr[qode]
+        sepptr[qode] = s += sepptr[qode]
+    end
+
+    for node in reverse(oneto(n))
+        qode = proj[node]
+
+        if ispositive(qode)
+            s = sepptr[qode]
+
+            for i in Iterators.reverse(neighbors(sep, node))
+                s -= one(E); septgt[s] = mark[i]
+            end
+
+            sepptr[qode] = s
+        else
+            qode = -qode
+        end
+
+        r = resptr[qode]
+
+        for i in reverse(neighbors(res, node))
+            mark[i] = r -= one(E)
+        end
+
+        resptr[qode] = r
+    end
+
+    for i in oneto(h)
+        work1[mark[i]] = order[i]
+    end
+
+    for i in oneto(h)
+        order[i] = work1[i]
+    end
+
+    m = sepptr[q + one(V)] - one(E)
+    qres = BipartiteGraph(h, q, h, resptr, oneto(h))
+    qsep = BipartiteGraph(h, q, m, sepptr, septgt)
+    atomtree = CliqueTree(SupernodeTree(Tree(qree), qres), qsep)
+    return order, atomtree
+end
+
 """
     treewidth([weights, ]tree::CliqueTree)
 
@@ -434,8 +662,8 @@ function treewidth_impl!(
         graph::AbstractGraph{V},
         index::AbstractVector{V},
     ) where {W, V, E}
-    @argcheck nv(graph) <= length(weights)
-    @argcheck nv(graph) <= length(wwork1)
+    @assert nv(graph) <= length(weights)
+    @assert nv(graph) <= length(wwork1)
     
     @inbounds for v in vertices(graph)
         wwork1[index[v]] = weights[v]
@@ -623,8 +851,8 @@ function treefill_impl!(
         graph::AbstractGraph{V},
         index::AbstractVector{V},
     ) where {W, V, E}
-    @argcheck nv(graph) <= length(weights)
-    @argcheck nv(graph) <= length(wwork1)
+    @assert nv(graph) <= length(weights)
+    @assert nv(graph) <= length(wwork1)
     
     @inbounds for v in vertices(graph)
         wwork1[index[v]] = weights[v]
@@ -709,6 +937,55 @@ function bestfill_impl!(
     return minindex
 end
 
+function separatorwidth(tree::CliqueTree{V}) where {V}
+    n = nov(residuals(tree)); weights = Ones{V}(n)
+    return separatorwidth(weights, tree)
+end
+
+function separatorwidth(weights::AbstractVector{W}, tree::CliqueTree) where {W}
+    maxwidth = zero(W)
+
+    @inbounds for bag in tree
+        width = zero(W)
+
+        for v in separator(bag)
+            width += weights[v]
+        end
+
+        maxwidth = max(maxwidth, width)
+    end
+
+    return maxwidth
+end
+
+function separatorwidth(weights::Ones{W}, tree::CliqueTree) where {W}
+    maxwidth = zero(W)
+
+    @inbounds for bag in tree
+        width = convert(W, length(separator(bag)))
+        maxwidth = max(maxwidth, width)
+    end
+
+    return maxwidth
+end
+
+function separatorwidth(graph; alg::PermutationOrAlgorithm = DEFAULT_ELIMINATION_ALGORITHM, snd::SupernodeType = DEFAULT_SUPERNODE_TYPE)
+    return separatorwidth(graph, alg)
+end
+
+function separatorwidth(graph, alg::PermutationOrAlgorithm, snd::SupernodeType)
+    perm, tree = cliquetree(graph, alg, snd)
+    return separatorwidth(tree)
+end
+
+function separatorwidth(weights::AbstractVector, graph; alg::PermutationOrAlgorithm = DEFAULT_ELIMINATION_ALGORITHM, snd::SupernodeType = DEFAULT_SUPERNODE_TYPE)
+    return separatorwidth(weights, graph, alg, snd)
+end
+
+function separatorwidth(weights::AbstractVector, graph, alg::PermutationOrAlgorithm, snd::SupernodeType)
+    perm, tree = cliquetree(graph, alg, snd)
+    return separatorwidth(weights[perm], tree)
+end
 
 """
     residual(tree::CliqueTree, i::Integer)
