@@ -68,6 +68,11 @@ The orderings computed by these algorithms induce minimum-width tree decompositi
 """
 abstract type EliminationAlgorithm end
 
+"""
+    MinimalAlgorithm <: EliminationAlgorithm
+"""
+abstract type MinimalAlgorithm <: EliminationAlgorithm end
+
 """
     PermutationOrAlgorithm = Union{
         AbstractVector,
@@ -310,7 +315,7 @@ julia> treewidth(graph; alg)
 
   - Rose, Donald J., R. Endre Tarjan, and George S. Lueker. "Algorithmic aspects of vertex elimination on graphs." *SIAM Journal on Computing* 5.2 (1976): 266-283.
 """
-struct LexM <: EliminationAlgorithm end
+struct LexM <: MinimalAlgorithm end
 
 """
     MCSM <: EliminationAlgorithm
@@ -344,7 +349,7 @@ julia> treewidth(graph; alg)
 
   - Berry, Anne, et al. "Maximum cardinality search for computing minimal triangulations of graphs." *Algorithmica* 39 (2004): 287-298.
 """
-struct MCSM <: EliminationAlgorithm end
+struct MCSM <: MinimalAlgorithm end
 
 """
     AMF <: EliminationAlgorithm
@@ -410,7 +415,7 @@ julia> treewidth(graph; alg)
 
 ### References
 
-  - Tinney, William F., and John W. Walker. "Direct solutions of sparse network equations by optimally ordered triangular factorization." *Proceedings of the IEEE* 55.11 (1967): 1801-1809.
+  - Ng, Esmond G., and Barry W. Peyton. "Fast implementation of the minimum local fill ordering heuristic." *CSC14: The Sixth SIAM Workshop on Combinatorial Scientific Computing.* 2014.
 """
 struct MF <: EliminationAlgorithm end
 
@@ -704,10 +709,10 @@ function ND{S}(
         imbalance::Integer = 130,
         scale::Integer = 1,
     ) where {S, A, D}
-    @argcheck ispositive(width)
-    @argcheck ispositive(level)
-    @argcheck ispositive(imbalance)
-    @argcheck ispositive(scale)
+    @assert ispositive(width)
+    @assert ispositive(level)
+    @assert ispositive(imbalance)
+    @assert ispositive(scale)
     return ND{S, A, D}(alg, dis, width, level, imbalance, scale)
 end
 
@@ -729,9 +734,9 @@ function NDS{S}(
         level::Integer = 6,
         imbalances::AbstractRange = 130:130,
     ) where {S, A, D}
-    @argcheck ispositive(width)
-    @argcheck ispositive(level)
-    @argcheck ispositive(first(imbalances))
+    @assert ispositive(width)
+    @assert ispositive(level)
+    @assert ispositive(first(imbalances))
     return NDS{S, A, D}(alg, dis, width, level, imbalances)
 end
 
@@ -971,9 +976,9 @@ julia> FilledGraph(tree2) # fewer edges
 
 ### References
 
-  - Blair, Jean RS, Pinar Heggernes, and Jan Arne Telle. "A practical algorithm for making filled graphs minimal." *Theoretical Computer Science* 250.1-2 (2001): 125-141.
+  - Heggernes, Pinar, and Barry W. Peyton. "Fast computation of minimal fill inside a given elimination ordering." *SIAM journal on matrix analysis and applications* 30.4 (2009): 1424-1444.
 """
-struct MinimalChordal{A <: PermutationOrAlgorithm} <: EliminationAlgorithm
+struct MinimalChordal{A <: PermutationOrAlgorithm} <: MinimalAlgorithm
     alg::A
 end
 
@@ -1097,7 +1102,7 @@ end
 """
     SafeRules{A, L, U} <: EliminationAlgorithm
 
-    SafeRules(alg::EliminationAlgorithm, lb::WidthOrAlgorithm, ub::EliminationAlgororithm)
+    SafeRules(alg::EliminationAlgorithm, lb::WidthOrAlgorithm)
 
     SafeRules()
 
@@ -1173,10 +1178,6 @@ function SafeRules(; kwargs...)
     return SafeRules(DEFAULT_ELIMINATION_ALGORITHM; kwargs...)
 end
 
-# deprecated
-const RuleReduction{A} = SafeRules{A, MMW{3}, MF}
-RuleReduction(alg) = SafeRules(alg, MMW(), MF())
-
 struct SimplicialRule{A <: EliminationAlgorithm} <: EliminationAlgorithm
     alg::A
     tao::Float64
@@ -1191,14 +1192,10 @@ function SimplicialRule(; kwargs...)
 end
 
 """
-    SafeSeparators{M, A} <: EliminationAlgorithm
+    SafeSeparators{A, M} <: EliminationAlgorithm
 
     SafeSeparators(alg::EliminationAlgorithm, min::PermutationOrAlgorithm)
 
-    SafeSeparators(alg::EliminationAlgorithm)
-
-    SafeSeparators()
-
 Apple an elimination algorithm to the atoms of an almost-clique separator decomposition. The algorithm
 `min` is used to compute the decomposition.
 
@@ -1219,17 +1216,13 @@ Apple an elimination algorithm to the atoms of an almost-clique separator decomp
   - Bodlaender, Hans L., and Arie MCA Koster. "Safe separators for treewidth." *Discrete Mathematics* 306.3 (2006): 337-350.
   - Tamaki, Hisao. "A heuristic for listing almost-clique minimal separators of a graph." arXiv preprint arXiv:2108.07551 (2021).
 """
-struct SafeSeparators{A <: EliminationAlgorithm, M <: PermutationOrAlgorithm} <: EliminationAlgorithm
+struct SafeSeparators{A <: EliminationAlgorithm, M <: MinimalAlgorithm} <: EliminationAlgorithm
     alg::A
     min::M
 end
 
-function SafeSeparators(alg::EliminationAlgorithm)
-    return SafeSeparators(alg, MinimalChordal())
-end
-
-function SafeSeparators()
-    return SafeSeparators(DEFAULT_ELIMINATION_ALGORITHM)
+function SafeSeparators(alg::EliminationAlgorithm=DEFAULT_ELIMINATION_ALGORITHM)
+    return SafeSeparators(alg, DEFAULT_MINIMAL_ALGORITHM)
 end
 
 """
@@ -1254,9 +1247,6 @@ function ConnectedComponents()
     return ConnectedComponents(DEFAULT_ELIMINATION_ALGORITHM)
 end
 
-# deprecated
-const ComponentReduction = ConnectedComponents
-
 struct Best{S, A <: NTuple{<:Any, PermutationOrAlgorithm}} <: EliminationAlgorithm
     algs::A
 end
@@ -1406,7 +1396,7 @@ function permutation(weights::AbstractVector, graph::AbstractGraph, alg::SAT{H})
 end
 
 function permutation(weights::AbstractVector, graph::AbstractGraph, alg::MinimalChordal)
-    return minimalchordal(graph, permutation(weights, graph, alg = alg.alg)...)
+    return mcs_etree(weights, graph, alg.alg)
 end
 
 function permutation(weights::AbstractVector, graph::AbstractGraph, alg::CompositeRotations)
@@ -1419,7 +1409,7 @@ function permutation(weights::AbstractVector, graph::AbstractGraph, alg::Compres
 end
 
 function permutation(weights::AbstractVector, graph::AbstractGraph, alg::SafeRules)
-    order = saferules(weights, graph, alg.alg, alg.lb, alg.ub, alg.tao)
+    order = saferules(weights, graph, alg.alg, alg.lb, alg.tao)
     return order, invperm(order)
 end
 
@@ -1429,7 +1419,7 @@ function permutation(weights::AbstractVector, graph::AbstractGraph, alg::Simplic
 end
 
 function permutation(weights::AbstractVector, graph::AbstractGraph, alg::SafeSeparators)
-    error()
+    return safeseparators(weights, graph, alg.alg, alg.min)
 end
 
 function permutation(weights::AbstractVector, graph::AbstractGraph, alg::ConnectedComponents)
@@ -1488,22 +1478,31 @@ end
 # The complexity is O(m + n), where m = |E| and n = |V|.
 function mcs(graph::AbstractGraph{V}, clique::AbstractVector{V} = oneto(zero(V))) where {V}
     j = one(V); n = nv(graph)
-    size = ones(V, n)
+    size = FVector{V}(undef, n)
     alpha = Vector{V}(undef, n)
 
+    @inbounds for v in vertices(graph)
+        size[v] = one(V)
+    end
+
     # construct bucket queue data structure
-    head = zeros(V, n + one(V))
-    prev = Vector{V}(undef, n)
-    next = Vector{V}(undef, n)
+    head = FVector{V}(undef, n + one(V))
+    prev = FVector{V}(undef, n)
+    next = FVector{V}(undef, n)
 
     function set(i)
-        return @inbounds DoublyLinkedList(view(head, i), prev, next)
+        @inbounds h = view(head, i)
+        return DoublyLinkedList(h, prev, next)
+    end
+
+    @inbounds for i in oneto(n + one(V))
+        empty!(set(i))
     end
 
     @inbounds prepend!(set(j), vertices(graph))
 
     # run algorithm
-    @inbounds for v in reverse(clique)
+    @inbounds for v in Iterators.reverse(clique)
         delete!(set(j), v)
         alpha[v] = n
         size[v] = one(V) - j
@@ -1558,9 +1557,9 @@ function rootls_impl!(
         root::V,
         tag::V,
     ) where {V}
-    @argcheck nv(graph) < length(xls)
-    @argcheck nv(graph) <= length(ls)
-    @argcheck nv(graph) <= length(marker)
+    @assert nv(graph) < length(xls)
+    @assert nv(graph) <= length(ls)
+    @assert nv(graph) <= length(marker)
 
     # initialization...
     n = nv(graph)
@@ -1602,9 +1601,9 @@ function fnroot_impl!(
         marker::AbstractVector{V},
         root::V,
     ) where {V}
-    @argcheck nv(graph) < length(xls)
-    @argcheck nv(graph) <= length(ls)
-    @argcheck nv(graph) <= length(marker)
+    @assert nv(graph) < length(xls)
+    @assert nv(graph) <= length(ls)
+    @assert nv(graph) <= length(marker)
 
     # initialize `tag`
     tag = one(V)
@@ -2166,7 +2165,7 @@ end
 # Jdrasil: A Modular Library for Computing Tree Decompositions
 # Bannach, Berndt, and Ehlers
 function sat(weights::AbstractVector{W}, graph::AbstractGraph{V}, upperbound::W, ::Val{H}) where {W, V, H}
-    @argcheck !isnegative(upperbound)
+    @assert !isnegative(upperbound)
     n = Int32(nv(graph))
 
     # compute a maximal clique
@@ -2384,10 +2383,13 @@ function maximalclique(weights::AbstractVector, graph::AbstractGraph, ::Val{H})
 
         # initialize assumption
         count = zero(Int32)
+        state = :sat
 
-        # Σ { var(i) : i } > count
-        solver[var[m - count]] = one(Int32)
-        state = solve!(solver)
+        if ispositive(m)
+            # Σ { var(i) : i } > count
+            solver[var[m - count]] = one(Int32)
+            state = solve!(solver)
+        end
 
         if state != :sat
             error("no solutions found")
@@ -2449,13 +2451,13 @@ function twins_impl!(
         graph::AbstractGraph{V},
         ::Val{S},
     ) where {V, S}
-    @argcheck nv(graph) < length(var)
-    @argcheck nv(graph) <= length(svar)
-    @argcheck nv(graph) <= length(flag)
-    @argcheck nv(graph) <= length(size)
-    @argcheck nv(graph) <= length(head)
-    @argcheck nv(graph) <= length(prev)
-    @argcheck nv(graph) <= length(next)
+    @assert nv(graph) < length(var)
+    @assert nv(graph) <= length(svar)
+    @assert nv(graph) <= length(flag)
+    @assert nv(graph) <= length(size)
+    @assert nv(graph) <= length(head)
+    @assert nv(graph) <= length(prev)
+    @assert nv(graph) <= length(next)
 
     n = nv(graph); m = zero(V)
 
@@ -2596,12 +2598,12 @@ function twins_impl!(
         ::Val{S},
         tao::T,
     ) where {V, S, T}
-    @argcheck nv(graph) < length(prev)
-    @argcheck nv(graph) <= length(next)
-    @argcheck nv(graph) <= length(adjmap)
-    @argcheck nv(graph) <= length(cosine)
-    @argcheck nv(graph) <= length(degree)
-    @argcheck zero(T) < tao <= one(T)
+    @assert nv(graph) < length(prev)
+    @assert nv(graph) <= length(next)
+    @assert nv(graph) <= length(adjmap)
+    @assert nv(graph) <= length(cosine)
+    @assert nv(graph) <= length(degree)
+    @assert zero(T) < tao <= one(T)
     n = nv(graph); nb = zero(V)
     list = DoublyLinkedList(head, prev, next)
 
@@ -2721,106 +2723,6 @@ function twins_impl!(
     return partition  
 end
 
-# A Practical Algorithm for Making Filled Graphs Minimal
-# Barry, Heggernes, and Telle
-# MinimalChordal
-function minimalchordal(graph::AbstractGraph{V}, order::AbstractVector, index::AbstractVector) where {V}
-    M = Graph(graph)
-    F = Vector{Vector{Tuple{V, V}}}(undef, nv(graph))
-
-    for (i, v) in enumerate(order)
-        F[i] = Tuple{V, V}[]
-        list = neighbors(M, v)
-        degree = eltypedegree(M, v)
-
-        for j in oneto(degree)
-            w = list[j]
-
-            if i < index[w]
-                for jj in (j + one(V)):degree
-                    ww = list[jj]
-
-                    if i < index[ww] && !has_edge(M, w, ww)
-                        add_edge!(M, w, ww)
-                        push!(F[i], (w, ww))
-                    end
-                end
-            end
-        end
-    end
-
-    Candidate = Set{Tuple{V, V}}()
-    Incident = V[]
-
-    for i in reverse(oneto(nv(graph)))
-        for (u, v) in F[i]
-            flag = true
-
-            for x in neighbors(M, u)
-                if index[x] > i && has_edge(M, x, v) && !has_edge(M, x, order[i])
-                    flag = false
-                    break
-                end
-            end
-
-            if flag
-                push!(Candidate, (u, v))
-                push!(Incident, u, v)
-            end
-        end
-
-        if !isempty(Candidate)
-            unique!(sort!(Incident))
-
-            n = length(Incident)
-            W = Graph{V}(n)
-
-            for j in oneto(n)
-                v = Incident[j]
-
-                for jj in (j + 1):n
-                    vv = Incident[jj]
-
-                    if (v, vv) ∉ Candidate
-                        add_edge!(W, j, jj)
-                    end
-                end
-            end
-
-            worder, windex = permutation(W; alg = MCSM())
-
-            for (j, v) in enumerate(worder)
-                list = neighbors(W, v)
-                degree = eltypedegree(W, v)
-
-                for k in oneto(degree)
-                    w = list[k]
-
-                    if j < windex[w]
-                        for kk in (k + 1):degree
-                            ww = list[kk]
-
-                            if j < windex[ww] && !has_edge(W, w, ww)
-                                add_edge!(W, w, ww)
-                                delete!(Candidate, (Incident[w], Incident[ww]))
-                            end
-                        end
-                    end
-                end
-            end
-
-            for (u, v) in Candidate
-                rem_edge!(M, u, v)
-            end
-
-            empty!(Candidate)
-            empty!(Incident)
-        end
-    end
-
-    return permutation(M; alg = MCS())
-end
-
 # Engineering Data Reduction for Nested Dissection
 # Ost, Schulz, Strash
 # Reduction 2 (Indistinguishable node reduction)
@@ -2854,8 +2756,8 @@ function compress_impl!(
         graph::AbstractGraph{V},
         ::Val{S},
     ) where {V, E, S}
-    @argcheck nv(graph) < length(outptr)
-    @argcheck ne(graph) <= length(outtgt)
+    @assert nv(graph) < length(outptr)
+    @assert ne(graph) <= length(outtgt)
 
     partition = twins_impl!(new, var, svar, flag, size, head, prev, next, graph, Val(S))
     project = svar; marker = size; tag = zero(V)
@@ -2916,8 +2818,8 @@ function compress_impl!(
         ::Val{S},
         tao::Number,
     ) where {V, E, S}
-    @argcheck nv(graph) < length(outptr)
-    @argcheck ne(graph) <= length(outtgt)
+    @assert nv(graph) < length(outptr)
+    @assert ne(graph) <= length(outtgt)
 
     partition = twins_impl!(head, prev, next, adjmap, cosine, degree, graph, Val(S), tao)
     project = adjmap; marker = cosine; tag = zero(V)
@@ -2968,8 +2870,8 @@ function compressweights(weights::AbstractVector{W}, project::AbstractGraph) whe
 end
 
 function compressweights_impl!(cmpweights::AbstractVector, weights::AbstractVector{W}, project::AbstractGraph) where {W}
-    @argcheck nv(project) <= length(cmpweights)
-    @argcheck nov(project) <= length(weights)
+    @assert nv(project) <= length(cmpweights)
+    @assert nov(project) <= length(weights)
 
     @inbounds for vcmp in vertices(project)
         wgt = zero(W)
@@ -2984,15 +2886,11 @@ function compressweights_impl!(cmpweights::AbstractVector, weights::AbstractVect
     return
 end
 
-function saferules(weights::AbstractVector, graph::AbstractGraph{V}, alg::EliminationAlgorithm, lb::WidthOrAlgorithm, ub::EliminationAlgorithm, tao::Number) where {V}
+function saferules(weights::AbstractVector, graph::AbstractGraph{V}, alg::EliminationAlgorithm, lb::WidthOrAlgorithm, tao::Number) where {V}
     n = nv(graph)
     width = lowerbound(weights, graph, lb)
     innerweights, innergraph, inject, project, innerwidth = compressreduce(pr4, weights, graph, width, tao)
-    innerorder, innerindex = permutation(innerweights, innergraph, ub)
-
-    if innerwidth < treewidth(innerweights, innergraph, (innerorder, innerindex))
-        innerorder, innerindex = permutation(innerweights, innergraph, alg)
-    end
+    innerorder, innerindex = permutation(innerweights, innergraph, alg)
 
     order = Vector{V}(undef, n); i = zero(V)
 
@@ -3254,7 +3152,6 @@ function Base.show(io::IO, ::MIME"text/plain", alg::SafeRules{A, L, U}) where {A
     println(io, " "^indent * "SafeRules{$A, $L, $U}:")
     show(IOContext(io, :indent => indent + 4), "text/plain", alg.alg)
     show(IOContext(io, :indent => indent + 4), "text/plain", alg.lb)
-    show(IOContext(io, :indent => indent + 4), "text/plain", alg.ub)
     println(io, " "^indent * "    tao: $(alg.tao)")
     return
 end
@@ -3292,6 +3189,11 @@ The default algorithm.
 """
 const DEFAULT_ELIMINATION_ALGORITHM = AMF()
 
+"""
+    DEFAULT_MINIMAL_ALGORITHM = MinimalChordal(AMF())
+"""
+const DEFAULT_MINIMAL_ALGORITHM = MinimalChordal(DEFAULT_ELIMINATION_ALGORITHM)
+
 """
     RCM = RCMGL
 
