@@ -23,12 +23,14 @@ struct Parent{V <: Integer, Prnt <: AbstractVector{V}} <: AbstractUnitRange{V}
     prnt::Prnt
 
     function Parent{V, Prnt}(nv::Integer, prnt::AbstractVector) where {V <: Integer, Prnt <: AbstractVector{V}}
-        @argcheck !isnegative(nv)
-        @argcheck nv <= length(prnt)
+        @assert !isnegative(nv)
+        @assert nv <= length(prnt)
         return new{V, Prnt}(nv, prnt)
     end
 end
 
+const FParent{V} = Parent{V, FVector{V}}
+
 function Parent{V, Prnt}(nv::Integer) where {V, Prnt}
     prnt = Prnt(undef, nv)
     return Parent(convert(V, nv), prnt)
@@ -53,27 +55,33 @@ function etree_impl!(
         tree::Parent{V},
         ancestor::AbstractVector{V},
         upper::AbstractGraph{V},
+        order::AbstractVector{V}=vertices(upper),
+        index::AbstractVector{V}=vertices(upper),
     ) where {V}
-    @argcheck nv(upper) == length(tree)
-    @argcheck nv(upper) <= length(ancestor)
+    @assert nv(upper) == length(tree)
+    @assert nv(upper) <= length(ancestor)
+    @assert nv(upper) <= length(order)
+    @assert nv(upper) <= length(index)
     n = nv(upper); parent = tree.prnt
 
     @inbounds for i in oneto(n)
         parent[i] = zero(V)
         ancestor[i] = zero(V)
 
-        for k in neighbors(upper, i)
-            r = k
+        for k in neighbors(upper, order[i])
+            r = index[k]
 
-            while !iszero(ancestor[r]) && ancestor[r] != i
-                t = ancestor[r]
-                ancestor[r] = i
-                r = t
-            end
+            if r < i
+                while !iszero(ancestor[r]) && ancestor[r] != i
+                    t = ancestor[r]
+                    ancestor[r] = i
+                    r = t
+                end
 
-            if iszero(ancestor[r])
-                ancestor[r] = i
-                parent[r] = i
+                if iszero(ancestor[r])
+                    ancestor[r] = i
+                    parent[r] = i
+                end
             end
         end
     end
@@ -87,8 +95,8 @@ function lcrs_impl!(
         child::AbstractVector{V},
         tree::Parent{V},
     ) where {V}
-    @argcheck length(tree) <= length(brother)
-    @argcheck length(tree) <= length(child)
+    @assert length(tree) <= length(brother)
+    @assert length(tree) <= length(child)
     root = zero(V)
 
     @inbounds for i in tree
@@ -108,13 +116,45 @@ function lcrs_impl!(
     return root
 end
 
+function lcrs_impl!(
+        brother::AbstractVector{V},
+        child::AbstractVector{V},
+        tree::Parent{V},
+        root::V,
+    ) where {V}
+    @assert length(tree) <= length(brother)
+    @assert length(tree) <= length(child)
+    @assert length(tree) >= root >= one(V)
+
+    @inbounds for i in tree
+        child[i] = zero(V)
+    end
+
+    skip = root; root = zero(V)
+
+    @inbounds for i in reverse(tree)
+        if i != skip
+            j = parentindex(tree, i)
+
+            if isnothing(j)
+                brother[i] = root; root = i
+            else
+                brother[i] = child[j]; child[j] = i
+            end
+        end
+    end
+
+    brother[skip] = root
+    return
+end
+
 function reverse!_impl!(
         graph::BipartiteGraph{V, V},
         tree::Parent{V},
     ) where {V}
-    @argcheck length(tree) == nv(graph) - one(V)
-    @argcheck length(tree) == nov(graph)
-    @argcheck length(tree) == ne(graph)
+    @assert length(tree) == nv(graph) - one(V)
+    @assert length(tree) == nov(graph)
+    @assert length(tree) == ne(graph)
 
     @inbounds for i in vertices(graph)
         ii = i + one(V)
@@ -171,15 +211,15 @@ function supcnt_impl!(
         lower::AbstractGraph{V},
         tree::Parent{V}
     ) where {W, V}
-    @argcheck nv(lower) <= length(wt)
-    @argcheck nv(lower) <= length(map0)
-    @argcheck nv(lower) <= length(inv0)
-    @argcheck nv(lower) <= length(inv1)
-    @argcheck nv(lower) <= length(fdesc)
-    @argcheck nv(lower) <= length(prev_p)
-    @argcheck nv(lower) <= length(prev_nbr)
-    @argcheck nv(lower) <= length(weights)
-    @argcheck nv(lower) <= length(tree)
+    @assert nv(lower) <= length(wt)
+    @assert nv(lower) <= length(map0)
+    @assert nv(lower) <= length(inv0)
+    @assert nv(lower) <= length(inv1)
+    @assert nv(lower) <= length(fdesc)
+    @assert nv(lower) <= length(prev_p)
+    @assert nv(lower) <= length(prev_nbr)
+    @assert nv(lower) <= length(weights)
+    @assert nv(lower) <= length(tree)
 
     n = nv(lower)
     postorder_impl!(map0, inv1, inv0, fdesc, tree)
@@ -298,11 +338,11 @@ function compositerotations_impl!(
         upper::AbstractGraph{V},
         clique::AbstractVector{V},
     ) where {V, E}
-    @argcheck nv(upper) <= length(alpha)
-    @argcheck nv(upper) <= length(order)
-    @argcheck nv(upper) <= length(index)
-    @argcheck nv(upper) <= length(fdesc)
-    @argcheck nv(upper) == nv(lower)
+    @assert nv(upper) <= length(alpha)
+    @assert nv(upper) <= length(order)
+    @assert nv(upper) <= length(index)
+    @assert nv(upper) <= length(fdesc)
+    @assert nv(upper) == nv(lower)
     n = nv(upper)
     etree_impl!(tree, fdesc, upper)
     reverse!_impl!(lower, upper)
@@ -384,10 +424,10 @@ function postorder_impl!(
         stack::AbstractVector{V},
         tree::Parent{V},
     ) where {V}
-    @argcheck length(tree) <= length(brother)
-    @argcheck length(tree) <= length(child)
-    @argcheck length(tree) <= length(index)
-    @argcheck length(tree) <= length(stack)
+    @assert length(tree) <= length(brother)
+    @assert length(tree) <= length(child)
+    @assert length(tree) <= length(index)
+    @assert length(tree) <= length(stack)
     num = zero(V)
 
     root = lcrs_impl!(brother, child, tree)
@@ -417,6 +457,48 @@ function postorder_impl!(
     return
 end
 
+function postorder_impl!(
+        brother::AbstractVector{V},
+        child::AbstractVector{V},
+        index::AbstractVector{V},
+        stack::AbstractVector{V},
+        tree::Parent{V},
+        root::V,
+    ) where {V}
+    @assert length(tree) <= length(brother)
+    @assert length(tree) <= length(child)
+    @assert length(tree) <= length(index)
+    @assert length(tree) <= length(stack)
+    @assert length(tree) >= root >= one(V)
+    num = zero(V)
+
+    lcrs_impl!(brother, child, tree, root)
+
+    function brothers(i::V)
+        @inbounds head = view(child, i)
+        return SinglyLinkedList(head, brother)
+    end
+
+    roots = SinglyLinkedList(Fill(root), brother)
+
+    @inbounds for i in roots
+        num += one(V); stack[num] = i
+    end
+
+    @inbounds for i in tree
+        j = stack[num]; num -= one(V)
+
+        while !isempty(brothers(j))
+            num += one(V); stack[num] = j
+            j = popfirst!(brothers(j))
+        end
+
+        index[j] = i
+    end
+
+    return
+end
+
 """
     postorder!(tree::Parent)
 
@@ -432,6 +514,16 @@ function postorder!(tree::Parent{V}) where {V}
     return index
 end
 
+function postorder!(tree::Parent{V}, root::V) where {V}
+    n = length(tree)
+    brother = Vector{V}(undef, n)
+    child = Vector{V}(undef, n)
+    index = Vector{V}(undef, n)
+    stack = Vector{V}(undef, n)
+    postorder!_impl!(brother, child, index, stack, tree, root::V)
+    return index
+end
+
 function postorder!_impl!(
         brother::AbstractVector{V},
         child::AbstractVector{V},
@@ -444,13 +536,26 @@ function postorder!_impl!(
     return
 end
 
+function postorder!_impl!(
+        brother::AbstractVector{V},
+        child::AbstractVector{V},
+        index::AbstractVector{V},
+        stack::AbstractVector{V},
+        tree::Parent{V},
+        root::V,
+    ) where {V}
+    postorder_impl!(brother, child, index, stack, tree, root)
+    invpermute!_impl!(stack, tree, index)
+    return
+end
+
 function firstdescendants_impl!(
         fdesc::AbstractVector{V},
         tree::Parent{V},
-        index::AbstractVector{V},
+        index::AbstractVector{V}=tree,
     ) where {V}
-    @argcheck length(tree) <= length(fdesc)
-    @argcheck length(tree) <= length(index)
+    @assert length(tree) <= length(fdesc)
+    @assert length(tree) <= length(index)
 
     @inbounds for i in tree
         fdesc[i] = index[i]
@@ -472,8 +577,8 @@ function invpermute!_impl!(
         tree::Parent{V},
         index::AbstractVector{V},
     ) where {V}
-    @argcheck length(tree) <= length(parent)
-    @argcheck length(tree) <= length(index)
+    @assert length(tree) <= length(parent)
+    @assert length(tree) <= length(index)
 
     @inbounds for i in tree
         j = parentindex(tree, i)
@@ -492,15 +597,16 @@ function invpermute!_impl!(
     return
 end
 
-@propagate_inbounds function setrootindex!(tree::Parent{I}, root::Integer) where {I}
-    @boundscheck checkbounds(tree, root)
-    i = zero(I); j = convert(I, root)
+function setrootindex!(tree::Parent{I}, root::Integer) where {I <: Integer}
+    return setrootindex!(tree, convert(I, root))
+end
+
+function setrootindex!(tree::Parent{I}, root::I) where {I <: Integer}
+    @assert root in tree
+    parent = tree.prnt; i = zero(I); j = root
 
     @inbounds while ispositive(j)
-        k = tree.prnt[j]
-        tree.prnt[j] = i
-        i = j
-        j = k
+        k = parent[j]; parent[j] = i; i = j; j = k
     end
 
     return tree
@@ -511,7 +617,7 @@ function Base.copy(tree::Parent)
 end
 
 function Base.copy!(dst::Parent, src::Parent)
-    @argcheck length(dst) == length(src)
+    @assert length(dst) == length(src)
     copyto!(dst.prnt, 1, src.prnt, 1, length(dst))
     return dst
 end
